{*******************************************************************************
  HiAsm stamp component builder v0.8 (autor - andrestudio <andrestudio@list.ru>)
  Component - ProgressDialog
  Version - 1.0
  Autor - andrestudio
  E-mail - andrestudio@list.ru
  Create - , 30.  2018, 17:21:56
  Description:   
*******************************************************************************}

unit hiProgressDialog;

interface

uses Windows,Kol,Share,Debug,ActiveX;

const

  IDA_OPERATION_ANIMATION = 1;
  // default normal progress dlg behavior
  PROGDLG_NORMAL        =  $00000000;
  // the dialog is modal to its hwndParent (default is modeless)
  PROGDLG_MODAL         =  $00000001;
  // automatically updates the "Line3" text with the "time remaining"
  // (you cant call SetLine3 if you passs this!)
  PROGDLG_AUTOTIME      =  $00000002;
  // we dont show the "time remaining" if this is set.
  // We need this if dwTotal < dwCompleted for sparse files
  PROGDLG_NOTIME        =  $00000004;
  // Do not have a minimize button in the caption bar.
  PROGDLG_NOMINIMIZE    =  $00000008;
  // Don't display the progress bar
  PROGDLG_NOPROGRESSBAR =  $00000010;

  // Time Actions (dwTimerAction)
  // Reset the timer so the progress will be calculated from now until
  // the first ::SetProgress() is called so
  PDTIMER_RESET     = $00000001;

  CLSID_ProgressDialog: TGUID = (
    D1:$f8383852; D2:$fcd3; D3:$11d1; D4:($a6, $b9, $0, $60, $97, $df, $5b, $d4));

  IID_IProgressDialog: TGUID = (
    D1:$EBBC7C04; D2:$315E; D3:$11D2; D4:($B6, $2F, $0, $60, $97, $DF, $5B, $D4));

type
  uint64 = int64;

type
 IProgressDialog = interface(IUnknown)
   ['{EBBC7C04-315E-11d2-B62F-006097DF5BD4}']
   function StartProgressDialog(
     hwndParent: HWND;
     punkEnableModless: IUnknown;
     dwFlags: DWORD;
     LpvResevered: Pointer): HRESULT; stdcall;
   function StopProgressDialog(): HRESULT; stdcall;
   function SetTitle(pwzTitle: PWideChar): HRESULT; stdcall;
   function SetAnimation(hInstAnimation: HMODULE; idAnimation: UINT): HRESULT; stdcall;
   function HasUserCancelled(): BOOL; stdcall;
   function SetProgress(dwCompleted: DWORD; dwTotal: DWORD): HRESULT; stdcall;
   function SetProgress64(ullCompleted: UInt64; ullTotal: UInt64): HRESULT; stdcall;
   function SetLine(dwLineNum: DWORD; pwzString: PWideChar;
     fCompactPath: BOOL; pvResevered: Pointer): HRESULT; stdcall;
   function SetCancelMsg(pwzCancelMsg: PWideChar; pvResevered: Pointer): HRESULT; stdcall;
   function Timer(dwTimerAction: DWORD; pvResevered: Pointer): HRESULT; stdcall;
 end;

type

 THIProgressDialog = class(TDebug)
   private
    PD: IProgressDialog;

   { Var methods variable }

   { Property variable  }
     {    }
     fTitle: string;
     {      }
     fMax: integer;
     {       }
     fLine1: string;
     {       }
     fLine2: string;
     { T      AutoTime = False}
     fLine3: string;
     {      }
     fStrCancel: string;
     {   }
     fModal: boolean;
     {     }
     fAutoTime: boolean;

     fStart: boolean;
     fFlags: DWord;

   public
   { Event points }
     _event_onCancel: THI_Event;

   { Data points}
     {      }
     _data_Max: THI_Event;
     {    }
     _data_Title: THI_Event;
     {        AutoTime = False}
     _data_Line3: THI_Event;
     {       }
     _data_Line2: THI_Event;
     {       }
     _data_Line1: THI_Event;
     {      }
     _data_StrCancel: THI_Event;

   { Events }
     {       }
     procedure onCancel;

     procedure Init;

   { Methods }
     {    }
     procedure _work_doPosition(var _Data: TData; Index: Word);

   { Methods from property }
     {      }
     procedure _work_doMax(var _Data: TData; Index: Word);
     {      }
     procedure _work_doStrCancel(var _Data: TData; Index: Word);
     {       }
     procedure _work_doLine1(var _Data: TData; Index: Word);
     {       }
     procedure _work_doLine2(var _Data: TData; Index: Word);
     {        AutoTime = False}
     procedure _work_doLine3(var _Data: TData; Index: Word);

   { Var procedure }

   { Property }
     {      }
     property _prop_Max: integer  read fMax  write fMax;
     {    }
     property _prop_Title: string  read fTitle  write fTitle;
     {       }
     property _prop_Line1: string  read fLine1  write fLine1;
     {       }
     property _prop_Line2: string  read fLine2  write fLine2;
     { T      AutoTime = False}
     property _prop_Line3: string  read fLine3  write fLine3;
     {      }
     property _prop_StrCancel: string  read fStrCancel  write fStrCancel;
     {   }
     property _prop_Modal: boolean  read fModal  write fModal;
     {     }
     property _prop_AutoTime: boolean  read fAutoTime  write fAutoTime;

  end;

implementation

function AnsiToWide(const s: String): PWideChar;
var
  len: integer;
  ws: WideString;
const
  codePage = 3;
begin
  Result := '';
  if s = '' then exit;
  len := MultiByteToWideChar(codePage, MB_PRECOMPOSED, PChar(@s[1]), -1, nil, 0);
  SetLength(ws, (len - 1){$ifdef F_P}*2{$endif});
  if len <= 1 then exit;
  MultiByteToWideChar(CodePage, MB_PRECOMPOSED, PChar(@s[1]), -1, PWideChar(@ws[1]), len);
  Result := PWideChar(ws);
end;

procedure THIProgressDialog.Init;
begin
  if _prop_Modal then
    fFlags := fFlags or PROGDLG_MODAL
  else
    fFlags := fFlags or PROGDLG_NORMAL;

  if _prop_AutoTime then
    fFlags := fFlags or PROGDLG_AUTOTIME
  else
    fFlags := fFlags or PROGDLG_NOTIME;

  CoCreateInstance(CLSID_ProgressDialog, nil, CLSCTX_INPROC_SERVER, IID_IProgressDialog, PD);
  PD.SetTitle(AnsiToWide(ReadString(_data_Empty,_data_Title,_prop_Title)));
  PD.SetLine(1, AnsiToWide(ReadString(_data_Empty,_data_Line1,_prop_Line1)), False, 0);
  PD.SetLine(2, AnsiToWide(ReadString(_data_Empty,_data_Line2,_prop_Line2)), False, 0);
  if not _prop_AutoTime then
    PD.SetLine(3, AnsiToWide(ReadString(_data_Empty,_data_Line3,_prop_Line3)), False, 0);
  PD.SetCancelMsg(AnsiToWide(ReadString(_data_Empty,_data_StrCancel,_prop_StrCancel)),nil);

  // Set the animation to play. .  avi   ?
  //PD.SetAnimation(hInstance, IDA_OPERATION_ANIMATION);
  PD.StartProgressDialog(ReadHandle, nil, fFlags, nil);
  fStart := True;
  PD.Timer(PDTIMER_RESET, nil);
  PD.SetProgress(0, fMax);
end;

{       }
procedure THIProgressDialog.onCancel;
begin
  _hi_onEvent(_event_onCancel);
end;

{    }
procedure THIProgressDialog._work_doPosition;
begin
  if (PD = nil) or (fStart = False) then Init;
  if PD.HasUserCancelled then onCancel;

  PD.SetProgress(ToInteger(_Data), fMax);
  if ToInteger(_Data) >= fMax then
  begin
    PD.StopProgressDialog;
    fStart := False;
  end;
end;

{      }
procedure THIProgressDialog._work_doMax;
begin
  _prop_Max := ReadInteger(_Data,_data_Max,_prop_Max);
end;

{      }
procedure THIProgressDialog._work_doStrCancel;
begin
  if (PD = nil) then
  begin
    _prop_StrCancel := ToString(_Data);
    Exit;
  end;
  PD.SetCancelMsg(AnsiToWide(ReadString(_Data,_data_StrCancel,_prop_StrCancel)),nil);
end;

{       }
procedure THIProgressDialog._work_doLine1;
begin
  if (PD = nil) then
  begin
    _prop_Line1 := ToString(_Data);
    Exit;
  end;
  PD.SetLine(1, AnsiToWide(ReadString(_Data,_data_Line1,_prop_Line1)), False, 0);
end;

{       }
procedure THIProgressDialog._work_doLine2;
begin
  if (PD = nil) then
  begin
    _prop_Line2 := ToString(_Data);
    Exit;
  end;
  PD.SetLine(2, AnsiToWide(ReadString(_Data,_data_Line2,_prop_Line2)), False, 0);
end;

{        AutoTime = False}
procedure THIProgressDialog._work_doLine3;
begin
  if (PD = nil) then
  begin
    _prop_Line3 := ToString(_Data);
    Exit;
  end;
  if not _prop_AutoTime then
    PD.SetLine(3, AnsiToWide(ReadString(_Data,_data_Line3,_prop_Line3)), False, 0);
end;


end.
